`timescale 1ns / 1ps
//******************************************************************************
// File Name            : i2c_24fcx_ctrl.v
//------------------------------------------------------------------------------
// Function             : i2c 24FCXXX controler
//                        
//------------------------------------------------------------------------------
// Designer             : yokomizo 
//------------------------------------------------------------------------------
// History
// -.-- 2010/10/14
//******************************************************************************
module i2c_24fcx_ctrl ( 
 clk, rstb, led_0,
 wr,rd,adr,wr_data,wr_bytes,rd_data,rd_data_en,rd_bytes, 
 tx_fifo_data,tx_fifo_data_en
);
  input clk;
  input rstb;   
  output led_0;
  output wr;
  output rd;
  output [6:0]adr;
  output [31:0]wr_data;
  output [2:0]wr_bytes;
  output [2:0]rd_bytes;
  input[31:0]rd_data;
  input  rd_data_en;
  output  [7:0] tx_fifo_data;    //tx fifo data 8bit
  output        tx_fifo_data_en; //tx fifo data enable 
 
  wire  led_0;
  reg  [25:0] main_cnt;

  reg wr;
  reg rd;
  reg [6:0]adr;
  reg [31:0]wr_data; 
  reg [2:0]wr_bytes; 
  wire [3:0]wr_be; 
  wire rd_data_en;
  wire [31:0]rd_data;
  reg [2:0]rd_bytes; 
  wire [3:0]rd_be; 
   
  reg [3:0] msg_cnt;  //tx fifo data 8bit
  reg [1:0] msg_type;
  wire [31:0]msg_data;
  wire [3:0]msg_be;
   
  reg  [7:0] tx_fifo_data;    //tx fifo data 8bit
  reg        tx_fifo_data_en; //tx fifo data enable 

  reg [15:0] mem_adr;
  reg [15:0] mem_dat;
   
   
function [7:0] hex2ascii;
  input [3:0] hex_data;
  begin
    if (hex_data<4'ha)
      hex2ascii = 8'h30 + hex_data;
    else
      hex2ascii = 8'h37 + hex_data;
  end
endfunction
   

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin 
    main_cnt <= 26'b0;
  end
  //else if (main_cnt == 26'd7999999) begin
  else if (main_cnt == 26'd39999999) begin
  //else if (main_cnt ==26'd5000000) begin
    main_cnt <= 26'b0;
    end
  else begin
    main_cnt <= main_cnt + 26'd1;
  end
   
assign led_0 = main_cnt[25];
 
// for 24FCXXX (EEPROM)

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0) begin
    wr <= 1'b0;
    wr_data <=32'h00000000;
    wr_bytes <= 3'd0; 
    adr <= 7'b1010000;  //EEPROM 24FC512 
    mem_adr <= 16'h0000; 
    mem_dat <= 16'h1000; 
    end
  else
    if ((main_cnt == 26'd500000)||
        (main_cnt ==26'd1000000)||
        (main_cnt ==26'd1500000)||
        (main_cnt ==26'd2000000))
      begin
        //f[^
        wr <= 1'b1;
        //[31:16]ɂ͏݃AhX,[15:0]ɂ͏݃f[^
        wr_data <= {mem_adr,mem_dat[15:8],mem_dat[7:0]};
        adr <= adr ;
        wr_bytes <= 3'd4;
        //݌ɃAhX+2,f[^-1
        mem_adr <= mem_adr + 16'h02;
        mem_dat <= mem_dat - 16'h01; 
      end
    else  if (main_cnt ==26'd2500000)
      begin
        //ǂݍݗpAhX
        wr <= 1'b1;
        wr_data <= {(mem_adr-16'h008),16'h0000};
        adr <= adr;
        wr_bytes <= 2'd2; 
      end
    else
      begin
        wr <= 1'b0;
        wr_data <= wr_data;
        adr <= adr;
        wr_bytes <= wr_bytes; 
      end

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      rd <= 1'b0; 
      rd_bytes <= 3'd0; 
    end   
  else   
    if (main_cnt ==26'd3000000)
      begin
        //f[^ǂݏo@1
        rd <= 1'b1;
        rd_bytes <= 3'd4; 
      end
    else if (main_cnt ==26'd3500000)
      begin
        //f[^ǂݏo2
        rd <= 1'b1;
        rd_bytes <= 3'd4; 
      end
    else
      begin   
        rd <= 1'b0;
        rd_bytes <= rd_bytes; 
      end   

// wr,rd byte enable
assign wr_be = (wr_bytes==3'd1)?4'b1000:    
               (wr_bytes==3'd2)?4'b1100: 
               (wr_bytes==3'd3)?4'b1110:
               (wr_bytes==3'd4)?4'b1111:4'b0000;
   
assign rd_be = (rd_bytes==3'd1)?4'b1000:    
               (rd_bytes==3'd2)?4'b1100: 
               (rd_bytes==3'd3)?4'b1110:
               (rd_bytes==3'd4)?4'b1111:4'b0000;
   

//re232c message

always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    msg_type <= 2'b0;
  else
    if (wr==1'b1)
      msg_type <= 2'b00;
    else if (rd_data_en==1'b1)
      msg_type <= 2'b01;
    else
      msg_type <= msg_type;
   
assign msg_data =(msg_type == 2'b00)? wr_data:
                 (msg_type == 2'b01)? rd_data:32'h00000000;
   
assign msg_be =( msg_type == 2'b00)? wr_be:
               ( msg_type == 2'b01)? rd_be:4'b0000;
   
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    msg_cnt <= 4'd0;
  else
    if ((wr==1'b1)||(rd_data_en==1'b1))
      msg_cnt <= 4'd1;
    else
      if (msg_cnt==4'd0)
        msg_cnt <= 4'd0;
      else
        msg_cnt <= msg_cnt + 4'd1;
    
always @ (posedge clk or negedge rstb )
  if (rstb==1'b0)
    begin
      tx_fifo_data <= 8'h00;
      tx_fifo_data_en <= 1'b0;
    end
  else
    case(msg_cnt)
      4'd0:begin
        tx_fifo_data <= 8'h00;
        tx_fifo_data_en <= 1'b0;
        end
      4'd1:begin
        if (msg_type==2'b00) 
          begin
            tx_fifo_data <= 8'h57; //"W"
            tx_fifo_data_en <= 1'b1;
          end
        else if (msg_type==2'b01) 
          begin
            tx_fifo_data <= 8'h52; //"R"
            tx_fifo_data_en <= 1'b1;
          end
        else  
          begin
            tx_fifo_data <= 8'h44; //"E"
            tx_fifo_data_en <= 1'b1;
          end
        end
      4'd2:begin
        tx_fifo_data <= 8'h5f; // "-"
        tx_fifo_data_en <= 1'b1;
        end
      4'd3:begin
        if (msg_type[1]==1'b0)
          begin 
             tx_fifo_data <= hex2ascii(adr[6:3]);
             tx_fifo_data_en <= 1'b1;
          end
        else
          begin
            tx_fifo_data <= 8'h5f; // "-"
            tx_fifo_data_en <= 1'b1;
          end
        end
      4'd4:begin
        if (msg_type[1]==1'b0)
          begin
            tx_fifo_data <= hex2ascii({adr[2:0],msg_type[0]});
            tx_fifo_data_en <= 1'b1;
          end
        else
          begin
            tx_fifo_data <= 8'h5f; // "-"
            tx_fifo_data_en <= 1'b1;
          end
        end        
      4'd5:begin
        tx_fifo_data <= 8'h5f; // "-"
        tx_fifo_data_en <= 1'b1;
        end
      4'd6:begin
        tx_fifo_data <=  hex2ascii(msg_data[31:28]);
        tx_fifo_data_en <= msg_be[3];
        end
      4'd7:begin
        tx_fifo_data <=  hex2ascii(msg_data[27:24]);
        tx_fifo_data_en <= msg_be[3];
        end
      4'd8:begin
        tx_fifo_data <=  hex2ascii(msg_data[23:20]);
        tx_fifo_data_en <= msg_be[2];
        end
      4'd9:begin
        tx_fifo_data <=  hex2ascii(msg_data[19:16]);
        tx_fifo_data_en <= msg_be[2];
        end
      4'd10:begin
        tx_fifo_data <=  hex2ascii(msg_data[15:12]);
        tx_fifo_data_en <= msg_be[1];
        end
      4'd11:begin
        tx_fifo_data <=  hex2ascii(msg_data[11:8]);
        tx_fifo_data_en <= msg_be[1];
        end
      4'd12:begin
        tx_fifo_data <=  hex2ascii(msg_data[7:4]);
        tx_fifo_data_en <= msg_be[0];
        end
      4'd13:begin
        tx_fifo_data <=  hex2ascii(msg_data[3:0]);
        tx_fifo_data_en <= msg_be[0];
        end
      4'd14:begin
        tx_fifo_data <= 8'h0a; // LF
        tx_fifo_data_en <= 1'b1;
        end
      default
        begin
          tx_fifo_data <= 8'h00;
          tx_fifo_data_en <= 1'b0;
        end
    endcase
              

   
endmodule






